'''
Created on 29/04/2009

@author: ben
'''

import pyglet
from pyglet.gl import *
import pyglet.window.key as KEY
import math
from copy import *
import pickle
import os

class TextTransform(object):
    "Encapsulates the transformation information for text positioning"
    def __init__(self):
        self.x = 0
        self.y = 0
        self.z = 0   
        self.camx = 0
        self.camy = 0
        self.camz = 100        
        self.rot_y = 0
        self.rot_x = 0
        self.width = 100
        self.height = 80
        self.aspect = 1
        
    def begin(self):
        glMatrixMode(GL_PROJECTION)
        glPushMatrix()
        glLoadIdentity()
        #gluPerspective(70,1,10,10000)
        gluPerspective(70,1,10,10000)
        glMatrixMode(GL_MODELVIEW)
        glPushMatrix()
        glLoadIdentity()
        gluLookAt(self.camx,self.camy,self.camz,0,0,0,0,1,0)
        glTranslatef(0,0,self.z)
        glRotatef(self.rot_y,0,1,0)
        glRotatef(self.rot_x,1,0,0)
        
    def end(self):
        glMatrixMode(GL_PROJECTION)
        glPopMatrix()
        glMatrixMode(GL_MODELVIEW)
        glPopMatrix()
        

class TextRenderer(object):
    '''
    Renders a text label with a given transformation.
    '''

    def __init__(self):
        
        self.layout = None
    
    def setText(self,text,size=6):
        self.document = pyglet.text.decode_html('<font face="BorisBlackBloxx" size="'+str(size)+'" color="#FFFFFF">'+text+'</font>')
        if self.layout is not None:
            self.setTransform(self.transform)
        
    def setTransform(self,transform):
        self.layout = pyglet.text.layout.TextLayout(self.document, transform.width/2, transform.height/2, multiline=True)
        self.layout.x = transform.x
        self.layout.y = transform.y        
        self.transform = transform
        
    def draw(self,width,height,debug=False):
        "needs width/height of window to compute correct aspect ratio"
        #self.transform.aspect = float(height)/width
        self.transform.begin()
        
        if debug:
            glColor3f(1,0,0)
            glRectf(self.layout.x,self.layout.y,self.layout.x+self.layout.width,self.layout.y+self.layout.height)        
        self.layout.draw()
        self.transform.end()

# TextRenderer editor
if __name__ == '__main__':
    
    # set up resource info
    pyglet.resource.path = ['../res']
    pyglet.resource.reindex()

    width, height = 800, 600
    window = pyglet.window.Window(width, height, visible=False)
    color_tan = (245/255.,231/255.,160/255.,1)    
    color_white = (1,1,1,1)
    apply(pyglet.gl.glClearColor,color_white)
    
    # specify a filename, if it exists already then it will load the data, 
    # otherwise it will create a new empty map to work with
    filename = "front.dat"
    
    try:
        file = pyglet.resource.file(filename,mode="r")
    except pyglet.resource.ResourceNotFoundException:
        print "Sorry, file ", filename, " doesn't exist!"
        exit(1)
    else:               
        md = pickle.load(file)
    bgimage = pyglet.resource.image(md.bgimage)
    
    # we can load older objects
    if "textAreaTransformation" not in md.__dict__ or md.textAreaTransformation==None:
        print "No text area transformation found! Creating a default one."
        md.textAreaTransformation = TextTransform()
        transform = md.textAreaTransformation
        transform.width = width/2
        transform.height = height/2
        transform.x = 0 # 72 - width/2
        transform.y = 0 # 331 - height/2
        transform.camz = 382 # 386
        transform.rot_y =  0 #79 #  451
    else:
        transform = md.textAreaTransformation

    tr = TextRenderer()
    text = '''Get off my LAWN!<br /> A game by Benjamin Porter.'''.upper()  
    tr.setText(text)
    tr.setTransform(transform)
    
    oldtransform = copy(transform)
    
    #bgimage = pyglet.resource.image("catroombg.png")
    
    instructions = ''' 
    KEY.X:'x',KEY.Y:'y',KEY.Z:'z'
    KEY.W:'width',KEY.H:'height'
    KEY.R:'rot'
    KEY.C:'camera',KEY.D:'dist' 
    '''
    print instructions
               
    
    t = 0
    dt = 1/60.0
#===============================================================================
#    
#    @window.event
#    def on_resize(w,h):
#        global width, height, aspectRatio
#        width, height = w, h 
#        aspectRatio = float(width)/height  
#        glViewport(0,0,width,height)     
#        glMatrixMode(GL_PROJECTION)
#        glLoadIdentity()
#        #gluOrtho2D(0,aspectRatio,0,1)
#        #gluOrtho2D(0,width,0,height)
#        gluPerspective(70,aspectRatio,10,10000)
#        glMatrixMode(GL_MODELVIEW)
#        return pyglet.event.EVENT_HANDLED # this is necessary
#===============================================================================
    
    editmode = None
    
    debug = True
    omx, omy = 0, 0
    mx, my = 0, 0
    @window.event
    def on_mouse_motion(x,y,dx,dy):
        global mx,my
        mx,my = x,y
        
        dmx = mx - omx
        dmy = my - omy
        
        if editmode=='x':
            transform.x = oldtransform.x + dmx
        if editmode=='y':
            transform.y = oldtransform.y + dmy
        if editmode=='z':
            transform.z = oldtransform.z + dmx
        if editmode=='camera':
            transform.camx = oldtransform.camx + dmx
            transform.camy = oldtransform.camy + dmy
        if editmode=='width':
            transform.width = oldtransform.width + dmx
        if editmode=='height':
            transform.height = oldtransform.height + dmy
        if editmode=='rot':
            transform.rot_y = oldtransform.rot_y + (1.0*dmx/width)*360
            transform.rot_x = oldtransform.rot_x + (1.0*dmy/height)*360
        if editmode=='dist':
            transform.camz = oldtransform.camz + dmy
            
        if editmode is not None:
            tr.setTransform(transform)
        
    @window.event
    def on_mouse_press(x,y,button,mod):
        global transform, editmode
        print(mx,my)
        
        if button==pyglet.window.mouse.LEFT:
            editmode = None
            tr.setTransform(transform)
    
        if button==pyglet.window.mouse.RIGHT:
            transform = copy(oldtransform)
            tr.setTransform(transform)
            editmode = None        
    
    @window.event
    def on_key_press(key,mod):
        global debug, editmode, oldtransform, omx, omy
        if key==KEY.Q:
            debug = not debug
        elif key==KEY.S:
            # save all current information
            print("Saving Text Transformation")
            md.textAreaTransformation = transform
            pickler = pickle.Pickler(open(os.path.join("..","res",filename),"w"))
            pickler.dump(md)
        else:
            if key==KEY._0 and editmode!=None:
                # zero this component
                if editmode=='x':
                    transform.x = 0
                if editmode=='y':
                    transform.y = 0
                if editmode=='z':
                    transform.z = 0
                if editmode=='camera':
                    transform.camx = 0
                    transform.camy = 0
                if editmode=='width':
                    transform.width = 0
                if editmode=='height':
                    transform.height = 0
                if editmode=='rot':
                    transform.rot_y = 0
                    transform.rot_x = 0
                if editmode=='dist':
                    transform.camz = 0
                omx = mx
                omy = my
                tr.setTransform(transform)
            else:
                oldmode = editmode
                editmode = {KEY.X:'x',KEY.Y:'y',KEY.Z:'z',KEY.W:'width',KEY.H:'height',KEY.R:'rot',KEY.C:'camera',KEY.D:'dist'}[key]
                oldtransform = copy(transform)
                omx = mx
                omy = my
                                
        
    @window.event
    def on_draw():
        global t
        t += dt
        
        window.clear()        
        
        glColor3f(1,1,1)
        bgimage.blit(0,0)
        tr.draw(debug,window.width,window.height)
                
    pyglet.clock.schedule_interval(lambda dt: None, dt)
    window.set_visible(True)
    pyglet.app.run()

    