'''
Created on 02/05/2009

@author: ben
'''

import globals
import pyglet
from characters import (Character,Player)
import random
from euclid import *
from roomrenderer import RoomRenderer
from misc import Dir
import math
import settings

class TheManIsControllable(Character):
    '''
    This man is for the opening sequence, and has a set of defined actions.
    '''
    walkingSpeed = 150 # 100 pixels per second
    dframes = 1#0.3

    def __init__(self,room,position):
        super(TheManIsControllable,self).__init__(room,position)
        
        if settings.fast_sprites:
            TheManIsControllable.dframes = .1
        else:
            TheManIsControllable.dframes = 1
            
        self.initSprites()        
        self.state = "idle"        
        self.targetLocation = Point2(0,0)        
        self.facingDirection = Dir.LEFT
        self.isWalking = False
        self.timeSince = 0
        self.targetting = False        
        self.targetAcquired = True
        self.show = True
    
    def setDrawState(self,state):
        "idle/frowning/etc.."
        self.drawstate = state
    
    def targetReached(self):
        "called when the target (if any) has been reached"
        self.targetAcquired = True
        self.targetting = False
        self.callback()            
        
    def draw(self,opacity=None):
        if self.show:
            spr = self.sprites[self.drawstate][0 if self.facingDirection==Dir.RIGHT else 1]
            if opacity is not None:
                spr.opacity = opacity*255
            if spr:
                spr.x = self.position.x
                spr.y = self.position.y
                spr.draw()

    def setNewTarget(self,room,position,callback):
        game = globals.game
        self.targetAcquired = False
        self.targetting = True
        self.targetRoom = room
        self.targetPortal = self.room.getPortal(game.world.planRoute(self.room,self.targetRoom)) 
        self.targetLocation = position
        self.callback = callback
                  
    def localUpdate(self,dt):
        # for now, just generate a random location every few seconds
        game = globals.game
        player = globals.player
        
        if self.targetting:
            # check if we are in the right room, if so, walk to the targetLocation
            if self.targetRoom==self.room:
                target = self.targetLocation
            else:
                target = self.targetPortal.position()
                
            # sprite stuff
            if (target.x - self.position.x)>0:
                self.startWalking(Dir.RIGHT)
            else:
                self.startWalking(Dir.LEFT)
                     
            # update position
            if self.isWalking:
                # find the vector from position to targetLocation
                direction = target - self.position
                distreq = direction.magnitude()
                maxdistwalkable = TheManIsControllable.walkingSpeed * dt
                if distreq<self.width/2:
                    # target acquired
                    if self.targetRoom==self.room:                    
                        self.targetReached()
                    else:
                        # must have hit a portal -- time to jump!
                        self.room.activatePortal(self,self.targetPortal)
                        # now set the new target
                        self.targetPortal = self.room.getPortal(game.world.planRoute(self.room,self.targetRoom))
                        game.roomsChanged(self) 
                else:    
                    direction.normalize()
                    dpos = direction * maxdistwalkable 
                    self.position = self.position + dpos
                    
    def startWalking(self,dir):
        self.isWalking = True
        self.walkingDirection = dir
        if self.walkingDirection==Dir.LEFT or self.walkingDirection==Dir.RIGHT:
            self.facingDirection = self.walkingDirection
        
        #select appropriate sprite
        self.drawstate = "walking"
    
    def stopWalking(self):
        self.isWalking = False
        self.drawstate = "idle"
        
    def initSprites(self):
        # load an image grid and turn it into a sprite        
        anchor_y = 30
        
        walkanim_img = pyglet.resource.image('man_walk.png')
        walkanim_seq = pyglet.image.ImageGrid(walkanim_img,1,4)
        for img in walkanim_seq:
            img.anchor_x = img.width/2
            img.anchor_y = anchor_y
            
        walkanim_imgl = pyglet.resource.image('man_walk_left.png')
        walkanim_seql = pyglet.image.ImageGrid(walkanim_imgl,1,4)
        for img in walkanim_seql:
            img.anchor_x = img.width/2
            img.anchor_y = anchor_y
        
        # XXX: Warning, decrease the interval too much and the system crashes!        
        walkanim_anim = walkanim_seq.get_animation(Player.dframes, True)
        walkanim_sprite = pyglet.sprite.Sprite(walkanim_anim)
        walkanim_animl = walkanim_seql.get_animation(Player.dframes, True)
        walkanim_spritel = pyglet.sprite.Sprite(walkanim_animl)
                
        baseimgright = pyglet.resource.image("man_stand.png")
        baseimgleft = pyglet.resource.image("man_stand.png",flip_x=True)
                                            
        baseimgright.anchor_x = baseimgright.width/2
        baseimgleft.anchor_x = baseimgleft.width/2
        baseimgright.anchor_y = anchor_y
        baseimgleft.anchor_y = anchor_y
        
        rightspr = pyglet.sprite.Sprite(baseimgright)
        leftspr = pyglet.sprite.Sprite(baseimgleft)        
        
        frownimgright = pyglet.resource.image("man_frowning.png")
        frownimgleft = pyglet.resource.image("man_frowning.png",flip_x=True)
                                            
        frownimgright.anchor_x = frownimgright.width/2
        frownimgleft.anchor_x = frownimgleft.width/2
        frownimgright.anchor_y = anchor_y
        frownimgleft.anchor_y = anchor_y
        
        frownrightspr = pyglet.sprite.Sprite(frownimgright)
        frownleftspr = pyglet.sprite.Sprite(frownimgleft)      
                
        self.sprites = {"idle":(rightspr,leftspr),
                        "frowning": (frownrightspr,frownleftspr),
                        "walking":(walkanim_sprite,walkanim_spritel)}
        self.drawstate = "idle"
        self.width = baseimgright.width/2
    
    def finish(self):
        for (v1,v2) in self.sprites.values():
            v1.delete()
            v2.delete()

class StartAnim(object):
    "handles the animation sequence at the start of the game"
    ballseqlength = 1
    
    def __init__(self):
        self.frontroom = globals.world.room("front")
        
        RoomRenderer.setRoom(self.frontroom)
    
        self.frontdoorpos = self.frontroom.getInteractionObject("door").position() 
       # self.ballpos = Point2(0,0)
           
        # load sprites
        self.theman = TheManIsControllable(self.frontroom,self.frontdoorpos)
        
        self.ball = pyglet.sprite.Sprite(pyglet.resource.image("soccerball.png"))
        self.ball.anchor_x = self.ball.width/2
        self.ball.x = -self.ball.width
        
        self.ballpos = self.frontroom.getInteractionObject("ballpos").position()        
        StartAnim.ex = self.ballpos.x
        StartAnim.ey = self.ballpos.y
        self.showball = True
        
        self.t = 0
        self.paused = False
        
        self.state = "start"
        self.theman.show = False
        #self.theman.setNewTarget(self.frontroom, Point2(random.random()*self.frontroom.width,100),self.callback)
    
        self.finished = False
        self.dsound = 0
    
    def pause(self, p = True):
        self.paused = p
    
    def finish(self):
        self.finished = True
        self.theman.finish()
        self.ball.delete()
    
    def callback(self):
        #self.theman.setNewTarget(self.frontroom, Point2(random.random()*self.frontroom.width,100),self.callback)
        pass
    
    def setState(self,state):
        self.state = state
    
    def update(self,dt):
        if not self.paused:
            self.t += dt
            if self.state=="start" and self.t > 2.0:
                self.state = "ball"
                self.t = 0
            elif self.state=="ball" and self.t > 1.0:
                # ball sequence             
                animT = (self.t-1)/StartAnim.ballseqlength
                h = self.setballpos(animT)
                if (self.t-1) > StartAnim.ballseqlength:
                    self.frontroom.getObject("flowers").show = False                
                    self.state = "cue man"
                    self.t = 0                            
            elif self.state=="cue man" and self.t > 2.0:
                self.theman.show = True
                self.t = 0
                self.state = "man appears"                
            elif self.state=="man appears" and self.t > 1.0:
                self.state = "man frowns"
                RoomRenderer.setText("Oi, who did this???".upper())
                self.theman.setDrawState("frowning")
                self.t = 0
            elif self.state=="man frowns" and self.t > 2.0:
                self.state = "man goes to ball"
                self.t = 0
                self.theman.setNewTarget(self.frontroom, self.ballpos, lambda: self.setState("got ball"))
            elif self.state=="got ball":
                self.t = 0
                self.theman.stopWalking()
                self.theman.setDrawState("frowning")
                self.state = "pick up ball"
                RoomRenderer.setText("I'll take this...".upper())
            elif self.state=="pick up ball" and self.t>1:      
                self.state = "man goes to door"
                self.theman.setNewTarget(self.frontroom, self.frontdoorpos,lambda: self.setState("man got to door"))
                self.t = 0
                self.showball = False
            elif self.state=="man got to door":
                self.theman.stopWalking()
                self.state = "man looking around"
                self.t = 0
                self.timeslooked = 0           
                self.theman.setDrawState("frowning")    
            elif self.state=="man looking around":
                if self.t > 0.5:
                    self.t = 0
                    self.theman.facingDirection = Dir.LEFT if self.timeslooked%2==0 else Dir.RIGHT
                    self.timeslooked += 1
                    if self.timeslooked>6:
                        self.state = "man went inside"                        
            elif self.state=="man went inside" and self.t > 1:
                self.theman.show = False
                self.state="nearly finished"
                RoomRenderer.setText("<br/>SLAM",size=7)
                globals.game.playSound('slam')
                self.t = 0
            elif self.state=="nearly finished" and self.t > 2:
                RoomRenderer.hidetext = True
                self.finished = True
            
            self.theman.localUpdate(dt)    
    
    def draw(self,dt):
        # render room
        RoomRenderer.render([self.theman],[self.ball] if self.showball else [])
        
        #render ball
        #if self.showball:
        #    self.ball.draw()
    
    sx = -100
    ex = 400
    sy = -50
    ey = 200
    yh = 150
    
    def setballpos(self,t):
        "parametric anim, not great but works"  
        self.ball.x = (StartAnim.ex - StartAnim.sx)*t + StartAnim.sx
        height = (1.05 - t) * StartAnim.yh*abs(math.sin(((4 + 4*t*t*t*t*t) - 1)*8*math.pi))
        self.ball.y = (StartAnim.ey - StartAnim.sy)*t + StartAnim.sy + height
        return height
    
    