'''
Created on 28/04/2009

@author: ben
'''
import pyglet
from pyglet.gl import *
from euclid import *
from misc import *
from things import Thing
from random import (random,choice)
import globals 
import settings

class Character(Thing):
    '''
    A character in the game. Each character is a singleton derived from this.
    '''
    def __init__(self,room,position):
        super(Character,self).__init__()
        self.room = room
        self.position = position
        self.localUpdateNeeded = True
        
    def moveto(self,to):
        "move the character to this position"
        self.position = to
        #print("move to: ",self.position)

    def isPlayer(self):
        "returns true if this character is the player"
        return False

class Player(Character):    
    walkingSpeed = 300 #400 #150 # pixels/per second
    dframes = 1 #.1 #1 #0.3
    
    def __init__(self,room,position):
        super(Player,self).__init__(room,position)        
        
        if settings.fast_sprites:
            Player.dframes = .1
        else:
            Player.dframes = 1
            
        self.initSprites()
        
        
        self.inventory = [] # a list of strings of obtained objects
        self.walkingDirection = None
        self.facingDirection = Dir.RIGHT
        self.isWalking = False        
        self.triedToMoveTo = None
    
    def addItem(self,item):
        self.inventory.append(item)
        
    def removeItem(self,item):
        self.inventory.remove(item)
        
    def hasItem(self,item):
        return item in self.inventory
        
    def moveto(self,to):
        super(Player,self).moveto(to)
        # informs the room that the player has moved, allowing events to be triggered
        self.room.moveto(self,to)        
        
    def draw(self,opacity=None):
        spr = self.sprites[self.state][0 if self.facingDirection==Dir.RIGHT else 1]
        #spr = self.sprites[self.state][0]
        if opacity is not None:
            spr.opacity = opacity*255
        if spr:
            # fill appropriate direction
            spr.x = self.position.x # - self.anchor_x
            spr.y = self.position.y #- self.anchor_y
            spr.draw()
            
        #self.img.blit(self.position.x,self.position.y)
#===============================================================================
#        
#        #DEBUG DRAW    
#        if True:
#            glPointSize(5.0)
#            glColor3f(1,0,0)
#            glBegin(GL_POINTS)
#            glVertex2f(self.position.x,self.position.y)
#            ttmt = self.triedToMoveTo
#            if ttmt:
#                glColor3f(1,1,0)
#                glVertex2f(ttmt.x,ttmt.y)
#            glEnd()
#            glColor3f(1,1,1)
#===============================================================================
        
    def localUpdate(self,dt):
        
        # update position
        if self.isWalking:                   
            dx = {Dir.DOWN:0,Dir.UP:0,Dir.LEFT:-1,Dir.RIGHT:1}[self.walkingDirection]
            dy = {Dir.DOWN:-1,Dir.UP:1,Dir.LEFT:0,Dir.RIGHT:0}[self.walkingDirection]
            oldpos = self.position
            dpos = Point2(dx,dy)*(Player.walkingSpeed*dt)
            newpos = oldpos + dpos
            newpos = Point2(newpos.x,newpos.y)
            self.triedToMoveTo = newpos
            # test if this position is reachable
            isReachable = True
            # the moveline has to be from the leading point, or from both if up/down
            startPosRel = Point2(0,0)
            if (self.walkingDirection!=Dir.DOWN and self.walkingDirection!=Dir.UP):
                if (dx>0): startPosRel = Point2(self.width/2,0)
                else: startPosRel = Point2(-self.width/2,0)
                endPt = oldpos + startPosRel + dpos
                endPt = Point2(endPt.x,endPt.y)                                
                isReachable = self.room.canMoveTo(oldpos,endPt)                
            else:
                endPt = oldpos + dpos + Point2(self.width/2,0)
                endPt = Point2(endPt.x,endPt.y)                
                isReachable = self.room.canMoveTo(oldpos,endPt)
                if isReachable:
                    endPt = oldpos + dpos + Point2(-self.width/2,0)
                    endPt = Point2(endPt.x,endPt.y)
                    isReachable = self.room.canMoveTo(oldpos,endPt)
            
            if isReachable:                
                self.moveto(newpos)                
            else:
                self.stopWalking()
    
    def startWalking(self,dir):
        self.isWalking = True
        self.walkingDirection = dir
        if self.walkingDirection==Dir.LEFT or self.walkingDirection==Dir.RIGHT:
            self.facingDirection = self.walkingDirection
        
        #select appropriate sprite
        self.state = "walking"
    
    def stopWalking(self):
        self.isWalking = False
        self.state = "idle"
        
    def isPlayer(self):
        return True
    
    def initSprites(self):
        # load an image grid and turn it into a sprite        
        anchor_y = 20
        
        walkanim_img = pyglet.resource.image('player_walk.png')
        walkanim_seq = pyglet.image.ImageGrid(walkanim_img,1,4)
        for img in walkanim_seq:
            img.anchor_x = img.width/2
            img.anchor_y = anchor_y
            
        walkanim_imgl = pyglet.resource.image('player_walk_left.png')
        walkanim_seql = pyglet.image.ImageGrid(walkanim_imgl,1,4)
        for img in walkanim_seql:
            img.anchor_x = img.width/2
            img.anchor_y = anchor_y
        
        # XXX: Warning, decrease the interval too much and the system crashes!
        
        walkanim_anim = walkanim_seq.get_animation(Player.dframes, True)
        walkanim_sprite = pyglet.sprite.Sprite(walkanim_anim)
        walkanim_animl = walkanim_seql.get_animation(Player.dframes, True)
        walkanim_spritel = pyglet.sprite.Sprite(walkanim_animl)
                
        baseimgright = pyglet.resource.image("player_stand.png")
        baseimgleft = pyglet.resource.image("player_stand.png",flip_x=True)
                                            
        baseimgright.anchor_x = baseimgright.width/2
        baseimgleft.anchor_x = baseimgleft.width/2
        baseimgright.anchor_y = anchor_y
        baseimgleft.anchor_y = anchor_y
        
        rightspr = pyglet.sprite.Sprite(baseimgright)
        leftspr = pyglet.sprite.Sprite(baseimgleft)
        
        self.sprites = {"idle":(rightspr,leftspr),"walking":(walkanim_sprite,walkanim_spritel)}
        self.state = "idle"
        self.width = baseimgright.width/2
        
class TheMan(Character):
    '''
    The man who stole the soccer ball.
    This guy acts quite simply. 
    He has a target location in mind, and walks directly there - through obstacles (no time left!)    
    He has some higher-level planning, which tells him how to get to a specific place in the house.
    '''
    walkingSpeed = 150 # 100 pixels per second
    dframes = 1#0.3

    def __init__(self,room,position):
        super(TheMan,self).__init__(room,position)
        
        if settings.fast_sprites:
            TheMan.dframes = .1
        else:
            TheMan.dframes = 1
            
        self.initSprites()
        
        
            
        self.state = "watchingtv"
        
        self.targetRoom = None
        self.targetLocation = Point2(0,0)
        self.targetPortal = None
        
        self.facingDirection = Dir.RIGHT
        self.isWalking = False
        self.timeSince = 0
        self.targetting = False        
        self.targetAcquired = True
        
        self.timeSinceLastSip = 0
        
    def notify(self,str):
        if self.state is not "sleeping":
            if str=="cat yelling":
                self.chase()
            elif str=="power off" and self.state=="watchingtv":
                self.state = "walktopowerbox"
                sideroom = globals.world.room("side")
                self.setNewTarget(sideroom,sideroom.getInteractionObject("fuse").position())        
        
    def finishedSwitch(self):
        if self.state=="switchingon":
            self.state = "walkbacktochair"
            tvroom = globals.world.room("loungeroom")
            self.setNewTarget(tvroom,tvroom.getObject("chair").position)
        
    def targetReached(self):
        "called when the target (if any) has been reached"
        self.targetAcquired = True
        
        if self.state=="walktopowerbox":
            globals.world.room("side").getInteractionObject("fuse").off = False            
            globals.game.notify("power on")
            self.state="switchingon"
            globals.game.scheduleOnce(lambda dt: self.finishedSwitch(), 2)
            self.targetting = False
        elif self.state=="walkbacktochair":
            # if the balls not there, unleash hell
            if globals.world.room("loungeroom").getObject("ball").show == False:
                self.chase()
            else:            
                self.state = "watchingtv"
                self.timeSinceLastSip = 0
                self.targetting = False
                self.stopWalking()
        elif self.state=="chase":
            globals.game.notify("caught")
            
            #keep chasing for now...
            self.targetAcquired = False                    
        else:
            self.stopWalking()
            self.targetting = False
            
    def chase(self):
        TheMan.walkingSpeed *= 2
        self.state = "chase"
        self.targetting = True
        
    def draw(self,opacity=None):
        spr = self.sprites[self.drawstate][0 if self.facingDirection==Dir.RIGHT else 1]
        if opacity is not None:
            spr.opacity = opacity*255
        if spr:
            spr.x = self.position.x
            spr.y = self.position.y
            spr.draw()

    def setNewTarget(self,room,position):
        game = globals.game
        self.targetAcquired = False
        self.targetting = True
        self.targetRoom = room
        self.targetPortal = self.room.getPortal(game.world.planRoute(self.room,self.targetRoom)) 
        self.targetLocation = position
        #if __debug__:
        #    print "man is going to ", self.targetRoom.name    
                  
    def localUpdate(self,dt):
        # for now, just generate a random location every few seconds
        game = globals.game
        player = globals.player
        
        if self.state=="sleeping":
            # Can't wake from this slumber
            pass
        
        else:
            # always check to see if the player is in the same room
            if self.state!="chase" and player.room==self.room:
                self.chase()
            
            if self.state=="chase":
                # update target location and position to the exact pos of player
                self.setNewTarget(player.room,player.position)       
            
            if self.targetting:
                # check if we are in the right room, if so, walk to the targetLocation
                if self.targetRoom==self.room:
                    target = self.targetLocation
                else:
                    target = self.targetPortal.position()
                
                # sprite stuff
                if (target.x - self.position.x)>0:
                    self.startWalking(Dir.RIGHT)
                else:
                    self.startWalking(Dir.LEFT)
                         
                # update position
                if self.isWalking:
                    # find the vector from position to targetLocation
                    direction = target - self.position
                    distreq = direction.magnitude()
                    maxdistwalkable = TheMan.walkingSpeed * dt
                    if distreq<self.width/2:
                        # target acquired
                        if self.targetRoom==self.room:                    
                            self.targetReached()
                        else:
                            # must have hit a portal -- time to jump!
                            self.room.activatePortal(self,self.targetPortal)
                            # now set the new target
                            self.targetPortal = self.room.getPortal(game.world.planRoute(self.room,self.targetRoom))
                            game.roomsChanged(self) 
                    else:    
                        direction.normalize()
                        dpos = direction * maxdistwalkable 
                        self.position = self.position + dpos
    
    def globalUpdate(self, dt):
        # only global update if not in room
        game = globals.game
        if self.room==game.room: return
        else:
            #update fast paced stuff
            self.localUpdate(dt)    
            #other slower state changes
            if self.state=="watchingtv":
                self.timeSinceLastSip += (dt * (random()+1)) #randomise a bit
                if self.timeSinceLastSip>4:
                    self.takeSip()
                    self.timeSinceLastSip = 0
    
    def sleep(self):
        self.state = "sleeping"
        self.drawstate = "sleeping"
        globals.game.notify("man fallen asleep")
        # now add an interaction object to the location he fell asleep, where you can get his keys
        sleepingbody = globals.world.room("loungeroom").getInteractionObject("sleepingbody")           
        sleepingbody.enabled = True
        sleepingbody.setPosition(self.position)
    
    def dead(self):
        self.state = "sleeping"
        self.drawstate = "dead"
        #globals.game.notify("man fallen asleep")
        # now add an interaction object to the location he fell asleep, where you can get his keys
        #sleepingbody = globals.world.room("loungeroom").getInteractionObject("sleepingbody")           
        #sleepingbody.enabled = True
        #sleepingbody.setPosition(self.position)
    
    def takeSip(self):
        globals.game.notify("take sip")
        drinkcontents = globals.world.room("loungeroom").getInteractionObject("drink").contents
        if "blue pill" in drinkcontents:
            self.sleep()            
                    
    def startWalking(self,dir):
        self.isWalking = True
        self.walkingDirection = dir
        if self.walkingDirection==Dir.LEFT or self.walkingDirection==Dir.RIGHT:
            self.facingDirection = self.walkingDirection
        
        #select appropriate sprite
        self.drawstate = "walking"
    
    def stopWalking(self):
        self.isWalking = False
        self.drawstate = "idle"
        
    def initSprites(self):
        # load an image grid and turn it into a sprite        
        anchor_y = 10
        
        walkanim_img = pyglet.resource.image('man_walk.png')
        walkanim_seq = pyglet.image.ImageGrid(walkanim_img,1,4)
        for img in walkanim_seq:
            img.anchor_x = img.width/2
            img.anchor_y = anchor_y
            
        walkanim_imgl = pyglet.resource.image('man_walk_left.png')
        walkanim_seql = pyglet.image.ImageGrid(walkanim_imgl,1,4)
        for img in walkanim_seql:
            img.anchor_x = img.width/2
            img.anchor_y = anchor_y
        
        # XXX: Warning, decrease the interval too much and the system crashes!
        
        walkanim_anim = walkanim_seq.get_animation(Player.dframes, True)
        walkanim_sprite = pyglet.sprite.Sprite(walkanim_anim)
        walkanim_animl = walkanim_seql.get_animation(Player.dframes, True)
        walkanim_spritel = pyglet.sprite.Sprite(walkanim_animl)
                
        baseimgright = pyglet.resource.image("man_stand.png")
        baseimgleft = pyglet.resource.image("man_stand.png",flip_x=True)
                                            
        baseimgright.anchor_x = baseimgright.width/2
        baseimgleft.anchor_x = baseimgleft.width/2
        baseimgright.anchor_y = anchor_y
        baseimgleft.anchor_y = anchor_y
        
        rightspr = pyglet.sprite.Sprite(baseimgright)
        leftspr = pyglet.sprite.Sprite(baseimgleft)
        
        sleepimg = pyglet.resource.image("man_sleeping.png")
        sleepimg.anchor_x = sleepimg.width/2
        sleepimg.anchor_y = baseimgleft.height/2 - sleepimg.height/2
        sleepspr = pyglet.sprite.Sprite(sleepimg)
        
        deadimg = pyglet.resource.image("man_dead.png")
        deadimg.anchor_x = deadimg.width/2
        deadimg.anchor_y = baseimgleft.height/2 - deadimg.height/2
        deadspr = pyglet.sprite.Sprite(deadimg)
                
        self.sprites = {"idle":(rightspr,leftspr),"walking":(walkanim_sprite,walkanim_spritel),
                        'sleeping':(sleepspr,sleepspr),'dead':(deadspr,deadspr)}
        self.drawstate = "idle"
        self.width = baseimgright.width/2
    
