'''
Created on 01/05/2009

@author: ben
'''

import pyglet
from pyglet import gl 
import pyglet.window.key as KEY
from euclid import *
#from copy import copy
import math, os
import random
import pickle
from misc import Dir
import world
from roomrenderer import RoomRenderer
import globals
from menu import Menu
import kooky
import anim
import stats
import settings

game_debug_mode = False

class Game(object):
    '''
    The controller of all.
    '''
    standardMessageWidth = 300
    standardMessageHeight = 400
    standardMessageX = 0
    standardMessageY = 0
    standardMessageFontSize = 5
    
    finalMessageWidth = 400
    finalMessageHeight = 600
    
    fullscreen = False
    speedyTime = 53    
        
    t = 0
    
    display_update = 1/60.0
    world_local_update = display_update
    world_global_update = world_local_update #.2
    
    def __init__(self):        
         globals.game = self
         self.width = 800   
         self.height = 600
         if self.fullscreen:
             self.window = pyglet.window.Window(fullscreen=True, visible=False)
             self.width = self.window.width
             self.height = self.window.height
             self.center = Point2(self.width/2,self.height/2)    
             Game.standardMessageX = self.center.x - Game.standardMessageWidth/2.
             Game.standardMessageY = self.center.y - Game.standardMessageHeight/2.  
         else:
             self.window = pyglet.window.Window(width=self.width,height=self.height,visible=False)
         
         # set up placement, etc..         
         Game.standardMessageX = (self.width - Game.standardMessageWidth)/2
         Game.standardMessageY = (self.height - Game.standardMessageHeight)/2
         
         # set up resource info
         pyglet.resource.path = ["res"]
         pyglet.resource.reindex()
         pyglet.resource.add_font('BorisBlackBloxx.ttf')
                  
         # music
         self.music = None
         self.playingMusic = False
         # I DECIDE NOT HAVE MUSIC...
#===============================================================================
#         try:
#            self.music = pyglet.media.Player()
#            self.music.eos_action = pyglet.media.Player.EOS_LOOP
#            self.music.queue(pyglet.resource.media("01 - Ambient A 13.47.mp3"))
#            self.music.play()
#            self.playingMusic = True
#         except IOError:
#            self.music = None
#         except pyglet.media.MediaFormatException:
#            print("Oops, you need AVBIN to play background music. Disabling for now..")
#            self.music = None
#===============================================================================
         
         # NO SOUND EFFECTS EITHER!!!
#===============================================================================
#         #sfx
#         self.sfx = {'gunshot': pyglet.resource.media("64346__Robinhood76__00672_gunshot_1.wav",streaming=False),
#                     'fart': pyglet.resource.media("26793_Walter_Odington_little_parp.wav",streaming=False),
#                     'chase': pyglet.resource.media("37986__sapht__Distorted_reverbed_kick_1.wav", streaming=False),
#                     'unlock': pyglet.resource.media("41349__datasoundsample__lock.wav", streaming=False),
#                     'hiss': pyglet.resource.media("4515__NoiseCollector__dragonhiss.wav", streaming=False),
#                     'slam': pyglet.resource.media("66964__evisneffo__evisneffo_Door_Slam.wav", streaming=False),
#                     'thud': pyglet.resource.media("43603_FreqMan_object_falls_5_.wav", streaming=False),
#                     }  
#===============================================================================
         
         self.fps_display = None
         if game_debug_mode:
             self.fps_display = pyglet.clock.ClockDisplay()
             #RoomRenderer.debug = True
        
         # setup event handling
         self.window.on_draw = self.on_draw      
         self.window.on_key_press = self.on_key_press
         self.window.on_key_release = self.on_key_release
        
         # schedule updates
         # pyglet.clock.schedule_interval(lambda dt: None, Game.display_update)
         pyglet.clock.schedule_interval(self.localUpdate, Game.world_local_update)
         pyglet.clock.schedule_interval(self.globalUpdate, Game.world_global_update)         
         
         # initialise rendering stuff
         self.init_renderer()
    
         # load world
         self.loadStats() 
         self.loadWorld()
         
         # start in shed for debug
         self.player.room = self.world.map.rooms["front"]
         startpos = self.player.room.getInteractionObject("start").position()
         #startpos = Point2(100,100)
         self.player.position = startpos
         
         self.room = self.player.room
         self.someoneAdjacent = False
         self.watching = self.player
         RoomRenderer.setRoom(self.player.room)
         
         # state stuff
         self.catYelling = False
         
         # effects
         self.roomFade = 1
         self.endingColour = None
         self._showEndingMessage = False
         self.kookyMode = False
         self.kookyDt = 0
         self.kookyMsg = ""
         self.endingDiscovered = None
         self.allEndingsDiscovered = False
         
         # set state
         self.state = None
         self.gametimer = 0
         self.setState("startanim")    
         #self.setState("playing")  
         self.showMenu = False   
    
    def toggleFullscreen(self):
         self.window.set_fullscreen(not self.window.fullscreen)      
         self.width = self.window.width
         self.height = self.window.height
         self.center = Point2(self.width/2,self.height/2)    
         Game.standardMessageX = self.center.x - Game.standardMessageWidth/2.
         Game.standardMessageY = self.center.y - Game.standardMessageHeight/2. 
         RoomRenderer.resize()        
    
    def playSound(self,sound):
        pass
        #self.sfx[sound].play()
    
    def exit(self):
        self.saveStats()
        exit(1)
        
    def loadStats(self):
        try:
            file = pyglet.resource.file("stats.dat",mode = "r")
            self.stats = pickle.load(file)
        except pyglet.resource.ResourceNotFoundException:
            self.stats = stats.Stats()
        settings.fast_sprites = self.stats.useFastSprites         
        self.allEndings = {"badminton":"Badminton ending",
                      "speedy gonzalez":"Speedy gonzalez ending",
                      "clean get away":"Clean get away ending",
                      "jail":"Jail ending",
                      "poo":"Stinky finale",
                      "sleeping beauty":"Sleeping beauty ending",
                      "murder":"Murder ending",
                      "void":"Into The Void ending"}
        self.undiscoveredEndings = []
        for e in self.allEndings.keys():  
            if e not in self.stats.endingsFound:
                self.undiscoveredEndings.append(self.allEndings[e]+" undiscovered")              
        
         
    def saveStats(self):
        pickler = pickle.Pickler(open(os.path.join("res","stats.dat"),"w"))
        pickler.dump(self.stats)
        
    def loadWorld(self):
        "Loads the world and game assets, TODO: make this threaded for a loading progress bar"
        self.world = world.World()
        self.player = self.world.player
        self.theman = self.world.theman
        globals.world = self.world
        globals.player = self.player
        globals.theman = self.world.theman
        
        self.startanim = anim.StartAnim()
        
        if self.stats.deskUnlocked:
            self.world.room("bedroom").getInteractionObject("desk").locked = False
                    
    def setState(self,state):
        if self.state=="startanim" and state!="startanim":
            self.startanim.finish()
            del self.startanim                
        self.state = state
        
    def updateState(self,dt):        
        if self.kookyMode:
            self.kookyDt += dt
            if self.kookyDt>1:
                self.kookyDt = 0
                self.nextKookyMessage()
                RoomRenderer.setText(self.kookyMsg)
        pass    
    
    def notify(self,str):
        "notify the game of some event..."
        if game_debug_mode:
            print "game event: ", str
        
        if str==None:
            if self.state=="startanim":
                self.startanim.pause(False)                                   
        if str=="Quit": self.exit()
        elif str=="Skip" and self.state=="startanim":
            self.setState("playing")
        elif str=="Disable Music":
            self.playingMusic = False
        elif str=="Enable Music":
            self.playingMusic = True   
        elif str=="toggle fullscreen":
            self.toggleFullscreen()
                
        for ch in self.world.characters:
                ch.notify(str)
        
        if str=="kooky shit":
            self.enableKookyMode()
        
        if str=="power off":
            self.world.room("loungeroom").getObject("tv").on = False
        
        if str=="power on":
            self.world.room("loungeroom").getObject("tv").on = True
            
        if str=="cat yelling":
            self.catYelling = True
            # start playing the cat yelling sound
            #print "Play sound: cat screech"
            
        if str=="man fallen asleep":
            if self.player.room.adjacentTo(self.theman.room):
                #TODO: Play sound: thud
                self.displayText("Thud",4,size=7)
                
        if str=="caught":
            if self.player.hasItem("handgun"):
                self.ending("manslaughter")
            else:            
                self.ending("nabbed")
        
        if str=="unlock desk":
            self.stats.deskUnlocked = True
                
    def query(self, listener, options, msg=None):
        "present a set of options to the player, and notify the listener of the result"
        # for now just use the console...
        # str = raw_input(options)
        self.listener = listener
        options.append("cancel")
        self.menu = Menu(msg,options,Game.standardMessageX,Game.standardMessageY, 
                         Game.standardMessageWidth, Game.standardMessageHeight, 
                         fontsize=Game.standardMessageFontSize,headercolour="#DDDDDD")
        self.showMenu = True
    
    def on_draw(self):
        Game.t += Game.display_update
        self.updateState(Game.display_update)            
        self.window.clear()
        
        ####### START ANIMATION ##########
        if self.state=="startanim":
          #draw animation sequence
          self.startanim.draw(Game.display_update)
          if self.startanim.finished: self.setState("playing")
          
        ####### END ANIMATION ##############                            
        elif self.state=="ending":
            # fade room out
            if self.roomFade > 0:
                self.roomFade = max(self.roomFade - Game.display_update*0.5,0) 
            else:
                self.roomFade = 0
                           
            RoomRenderer.render([])
            RoomRenderer.begin()
            r,g,b = self.endingColour
            gl.glColor4f(r,g,b,1-self.roomFade)
            gl.glRectf(0,0,self.width,self.height)            
            # draw characters over top
            gl.glColor4f(1,1,1,1)
            for ch in self.world.characters:
                if ch.room == self.room:
                    ch.draw((self.roomFade+1.)/2.)
            RoomRenderer.end()
            
            if self._showEndingMessage:
                if self.allEndingsDiscovered:
                    Menu("You have discovered all the endings. Congratulations. Thanks for playing. <br />BP 020509".upper(),[],(self.width-Game.finalMessageWidth)/2.,(self.height-Game.finalMessageHeight)/2.,Game.finalMessageWidth,Game.finalMessageWidth,0,fontsize=5,headercolour="#333333").draw(Game.display_update)
                else:
                    msg = Menu(self.endingMessage.upper(),[],(self.width-Game.finalMessageWidth)/2.,(self.height-Game.finalMessageHeight)/2.,Game.finalMessageWidth,Game.finalMessageWidth,0,fontsize=5)
                    msg.draw(Game.display_update)
                    
                    if self.endingDiscovered is not None:
                        Menu(self.endingDiscovered.upper(),[],(self.width-Game.finalMessageWidth)/2.,(self.height-Game.finalMessageHeight)/2.-msg.ch,Game.finalMessageWidth,Game.finalMessageWidth,0,fontsize=4,headercolour="#000000").draw(Game.display_update)
        ########### OTHERWISE, NORMAL ###################    
        if self.state=="playing":       
            self.gametimer += Game.display_update
            RoomRenderer.render(self.world.characters)
        
        if self.showMenu:
            if self.menu:
                #gl.glViewport(0,self.width,0,self.height)
                self.menu.draw(Game.display_update)
        
        if self.fps_display: self.fps_display.draw()
       
    def on_key_press(self,key,mod):
        if self.showMenu:
            if key==KEY.UP:
                self.menu.up()
            elif key==KEY.DOWN:
                self.menu.down()
            elif key==KEY.SPACE or key==KEY.ENTER:
                selection = self.menu.select()
                if selection=="cancel":
                    self.listener.notify(None)
                else:
                    self.listener.notify(selection)
                self.showMenu = False
                self.menu = None
            elif key==KEY.ESCAPE:
                self.listener.notify(None)
                #self.setState("playing")
                self.showMenu = False
                self.menu = None
        elif self.state=="startanim":
            if key==KEY.ESCAPE:
                self.startanim.pause(True)
                self.query(self,["Skip","Quit"])
        elif self.state=="playing":
            if game_debug_mode:
                if key==KEY.D:
                    RoomRenderer.debug = not RoomRenderer.debug
                    if RoomRenderer.debug:
                        self.displayText("DEBUG MODE ON", 2.0)
                    else:
                        self.displayText("DEBUG MODE OFF", 2.0)
                if key==KEY.T:
                    self.displayText("THIS IS SOME TEXT. IT IS USED TO TEST THE TEXT TRANSFORMATIONS", 2.0)
                if key==KEY.W:
                    if self.watching is self.player:
                        self.watching = self.theman
                    elif self.watching is self.theman:
                        self.watching = self.player
                    self.roomsChanged(self.watching)
                if key==KEY.S:
                    #put man to sleep
                    self.theman.sleep()
            
            if key==KEY.ESCAPE:
                self.query(self,["toggle fullscreen"]+(["Disable Music" if self.playingMusic else "Enable Music"] if self.music else []) + ["Quit"])
            elif key==KEY.SPACE:
                self.player.room.use()
            else:
                walkingDirection = Dir.dirFromKey(key)
                if walkingDirection==None:
                    self.player.stopWalking()
                else:
                    self.player.startWalking(walkingDirection)            
        elif self.state=="ending":
            if key==KEY.ESCAPE:
                self.exit()
                                    
    def on_key_release(self,key,mod):
        # NOTE: stop walking, even if in menu mode
        if self.player.isWalking and self.player.walkingDirection==Dir.dirFromKey(key):
            self.player.stopWalking()
        pass
    
    def nextKookyMessage(self):        
        self.kookyMsg = kooky.randomMessage()
    
    def enableKookyMode(self):
        self.kookyMode = True        
        self.world.room("hallway").openSecretDoor() 
        RoomRenderer.kookyMode = True
    
    def roomsChanged(self,person):
        "update adjacent room details"
        adj = self.player.room.adjacentTo(self.theman.room)
        
        if person==self.watching:
            RoomRenderer.setRoom(self.watching.room)
            self.room = self.watching.room
        else:
            pass
        
        if person==self.theman:
            if not self.someoneAdjacent and adj:
                self.displayText("Someone just entered the next room", 1)
            elif self.player.room is not person.room and self.someoneAdjacent and not adj:
                self.displayText("I think they've gone", 1)
        elif person==self.player:
            if not self.someoneAdjacent and adj:
                self.displayText("someone's in the next room", 1)
        
        
        self.someoneAdjacent = adj
                    
    def playerChangedRoom(self):
        "called when the player switches room"        
        self.roomsChanged(self.player)
    
    def localUpdate(self,dt):
        if self.state=="startanim":
            self.startanim.update(dt)
        elif self.state!="ending":
            self.world.localUpdate(dt,self.room)
        pass
        
    def globalUpdate(self,dt):
        if self.state is not "startanim" and self.state is not "ending":
            self.world.globalUpdate(dt)
        pass
    
    def removeText(self, dt):
        RoomRenderer.hidetext = True        
        
    def scheduleOnce(self, function, delay):
        "wait/block for some duration"
        pyglet.clock.schedule_once(function, delay)
        
    def displayText(self,text,duration=None,size=None):
        "display some deformed text for some duration"
        #if game_debug_mode:
        #    print "MSG (" + str(duration if duration is not None else 0) + "s): " + text
        
        RoomRenderer.hidetext = False
        if size is not None:
            RoomRenderer.setText(text.upper(),size)
        else:
            RoomRenderer.setText(text.upper())
       
        pyglet.clock.unschedule(self.removeText)
        if duration is not None:
            pyglet.clock.schedule_once(self.removeText, duration)
        
    def showEndingMessage(self):
        self._showEndingMessage = True  
        
    def showEndingDiscovered(self,msg):
        self.endingDiscovered = msg
    
    def showAllEndingsDiscovered(self):
        self.allEndingsDiscovered = True
        
    def ending(self,endingtype):
        "start the ending sequence of this ending type"
        if game_debug_mode:
            print "ending:", endingtype
        
        self.finishtime = self.gametimer
        endingsFound = self.stats.endingsFound
        
        self.setState("ending")
        #self.state = "ending"
        
        # determine the ending type
        if endingtype=="went home" and not self.player.hasItem("ball"):
            self.endingColour = (224/255.,184/255.,180/255.)
            self.endingMessage = "You decide not to break in and take up badminton. It's fun."
            if "badminton" not in endingsFound:
                endingsFound.append("badminton")
                self.showEndingDiscovered("Badminton ending discovered")
            
        elif endingtype=="out front door" or (endingtype=="went home" and self.player.hasItem("ball")):
            self.endingColour = (200/255.,200/255.,210/255.)
            self.endingMessage = "You run out of the house, triumphant in your rescue. It only took you " + str(int(self.finishtime)) + " seconds."    
            
            if self.stats.bestTime is None or self.stats.bestTime>self.finishtime:
                self.endingMessage += "A new record, "
                                                    
                if self.finishtime > Game.speedyTime:
                    self.endingMessage += "but it can be done quicker."
                elif self.finishtime <= Game.speedyTime:
                    if "speedy gonzalez" not in endingsFound:
                        endingsFound.append("speedy gonzalez")                    
                        self.showEndingDiscovered("Speedy gonzalez ending found, Arreebah!")
                                
                self.stats.bestTime = self.finishtime   
            
            if self.endingDiscovered==None and "clean get away" not in endingsFound:
                endingsFound.append("clean get away")
                self.showEndingDiscovered("\"Clean get away\" ending found.")                
                  
        elif endingtype=="nabbed":
            self.endingColour = (220/255.,220/255.,160/255.)
            self.endingMessage = "You've been caught! It's juvy for you sonny!"
            
            if "jail" not in endingsFound:
                endingsFound.append("jail")
                self.showEndingDiscovered("Jail ending found. That's not a good thing...")                
            
        elif endingtype=="poo":
            self.endingColour = (153/255.,120/255.,44./255)
            self.endingMessage = "Eww, you got poo on your hands. You run home crying. Maybe you should've worn gloves."
            
            if "poo" not in endingsFound:
                endingsFound.append("poo")
                self.showEndingDiscovered("Stinky finale found.")
            
        elif endingtype=="fell asleep":
            self.endingColour = (164./255,210./255,231/255.)
            self.endingMessage = "You fall into a deep sleep... and never awaken."
            
            if "sleeping beauty" not in endingsFound:
                endingsFound.append("sleeping beauty")
                self.showEndingDiscovered("Sleeping Beauty ending discovered.")   
            
        elif endingtype=="manslaughter":
            #self.endingColour = (180/255.,70/255.,54/255.)  #blood colour
            self.endingColour = (190/255.,80/255.,64/255.)  #blood colour
            self.endingMessage = "A struggle ... a loaded gun ... A dead man."
            #TODO: PLAY GUNSHOT
            self.theman.dead()
            
            if "murder" not in endingsFound:
                endingsFound.append("murder")
                self.showEndingDiscovered("Murder ending discovered.")           
            
        elif endingtype=="into the void":
            self.endingColour = (0,0,0)
            self.endingMessage = "The blackness is enticing, alluring, but cold. You soon forget who you are and why you entered pyweek."
            
            if "void" not in endingsFound:
                endingsFound.append("void")
                self.showEndingDiscovered("Void ending discovered.")
        
        if self.endingDiscovered is not None:
            if len(endingsFound)==len(self.allEndings.keys()):
                self.scheduleOnce(lambda dt: self.showAllEndingsDiscovered(),4.0)
        
        self.scheduleOnce(lambda dt: self.showEndingMessage(), 2.0)
        
        # clear screen etc ...
        self.removeText(0)
        
    def start(self):
        self.window.set_visible(True)
        try:
            pyglet.app.run()
        except TypeError:
            print "Unfortunately, a problem has occurred with the graphics engine \"pyglet\".\nI am disabling some features -- please run the game again."
            self.stats.useFastSprites = False
            self.exit()            
        
    def init_renderer(self):
        gl.glEnable(gl.GL_BLEND)
        gl.glBlendFunc(gl.GL_SRC_ALPHA, gl.GL_ONE_MINUS_SRC_ALPHA)  

if __name__ == '__main__':
    game = Game()
    game.start()    
    
