'''
Created on 27/04/2009

Displays a bitmap, and allow informational annotations...

@author: ben
'''

import pyglet
from pyglet.gl import *
from euclid import *
from copy import copy
import pyglet.window.key as KEY
import math, os
from MapData import (MapData,InteractionArea)
import pickle
from TextRenderer import TextTransform
from misc import *

if __name__ == '__main__':
    # set up resource info
    pyglet.resource.path = ['../res']
    pyglet.resource.reindex()
    
    # current edit mode (walkarea,layerspec,etc..)
    editMode = "none"
    editInfo = Bunch() #editmode specific information
    
    # specify a filename, if it exists already then it will load the data, 
    # otherwise it will create a new empty map to work with
    room = "back"
    filename = room + ".dat"
    
    try:
        file = pyglet.resource.file(filename,mode="r")
    except pyglet.resource.ResourceNotFoundException:
        md = MapData()
        md.bgimage = room + "bg.png"
        
        #md.bgimage = raw_input("What is the bgimage:")
    else:               
        md = pickle.load(file)
        md.output()
        
    bgimage = pyglet.resource.image(md.bgimage)
        
    width, height = bgimage.width, bgimage.height
    window = pyglet.window.Window(width, height, visible=False)
    glEnable(GL_BLEND)
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
    
    
    
#===============================================================================
#    # we can load older objects
#    if "objects" not in md.__dict__:
#        md.objects = []
#===============================================================================
    
    # cache the placeholders
    objimages = {}    
    for obj in md.objects:
        objimages[obj.img] = pyglet.resource.image(obj.img)
    
    t = 0
    dt = 1/60.0

    mx, my = 0, 0
    @window.event
    def on_mouse_motion(x,y,dx,dy):
        global mx,my
        mx,my = x,y
        
    @window.event
    def on_mouse_press(x,y,button,mod):
        global editInfo,editMode
        pos = Point2(x,y)
        if editMode=="walkarea":
            if editInfo.mode=='first':
                editInfo.points.append((x,y))
                editInfo.mode='adding'
            else:
                # check to see if the point is near the first point
                fx,fy = editInfo.points[0]
                if (pos - Vector2(fx,fy)).magnitude_squared()<20:
                    editInfo.mode='finished'
                    editMode = 'none'
                    md.walkArea = copy(editInfo.points)
                else:
                    editInfo.points.append((x,y))
        elif editMode=="interactionarea":
            if editInfo.mode=="select":
                #find the interaction area close to this point
                for i in md.interactionAreas:
                    icirc = Circle(Point2(i.x,i.y),1.0*i.r)
                    if (pos.intersect(icirc)):
                        editInfo.mode='edit'
                        editInfo.selected = i
                        print("selected interaction area")
                        break                    
            elif editInfo.mode=='edit':
                assert editInfo.selected
                s = editInfo.selected
                if editInfo.component=='r':
                    s.r = pos.distance(Point2(s.x,s.y))
                elif editInfo.component=='p':
                    s.x,s.y = x,y     
#===============================================================================
#        elif editMode=="walkarealayer":
#            if editInfo.mode=='select':
#                # select region
#                pass
#            elif editInfo.mode=='adding':
#                # check to see if the point is near the first point
#                if len(editInfo.selected[0])>0:
#                    fx,fy = editInfo.selected[0][0]
#                    if (pos - Vector2(fx,fy)).magnitude_squared()<20:
#                        editInfo.mode='finished'
#                        editInfo.selected[1] = int(raw_input("layer number:"))                        
#                    else:
#                        editInfo.selected[0].append((x,y))
#                else:               
#                    editInfo.selected[0].append((x,y))
#===============================================================================
        elif editMode=="objects":
            if editInfo.mode=="select":
                for obj in md.objects:
                    if (Point2(obj.position[0],obj.position[1]).distance(pos)<40):
                        print("selected object " + obj.name)
                        editInfo.mode=='edit'
                        editInfo.selected = obj
                        break
            elif editInfo.mode=='edit':
                if editInfo.component=="pos":
                    editInfo.selected.position = pos.x,pos.y
                elif editInfo.component=="pivot":
                    p = editInfo.selected.position
                    editInfo.selected.pivot = (pos.x - p[0],pos.y - p[1])                
                    
    @window.event
    def on_key_press(key,mod):
        global editMode, editInfo
        if key==KEY.W:
            print("edit: walkareas")
            editMode = "walkarea"
            print("editting walk area")
            editInfo = Bunch(mode='first',points=[])
#===============================================================================
#        if key==KEY.L:
#            print("edit: walkarealayer")
#            editMode = "walkarealayer"
#            print("editting walk area layer spec")
#            editInfo = Bunch(mode='select',selected=None)
#        if editMode=="walkarealayer":
#            if key==KEY.N:
#                # new layer spec
#                md.walkAreaLayerSpecs.append([[],0])
#                editInfo.selected = md.walkAreaLayerSpecs[-1]
#                editInfo.mode = 'adding'
#===============================================================================
        
        if editMode=="none" and key==KEY.I:
            print("edit: interactionarea")
            editMode = "interactionarea"
            editInfo = Bunch(mode='select',selected=None,component=None) 
        if editMode=="interactionarea" and editInfo.mode=='edit':
            if key==KEY.R:
                editInfo.component = 'r'
            elif key==KEY.P:
                editInfo.component = 'p'
            elif key==KEY.I:
                editInfo.component = None
                id = raw_input("enter id:")
                print("You entered (" + id + ")")
                editInfo.selected.id = id            
                editInfo.mode = 'select'    
        elif editMode=="interactionarea":
            if key==KEY.N:
                md.interactionAreas.append(InteractionArea())
                
        if key==KEY.O:
            print("edit: objects")
            editMode = "objects"
            editInfo = Bunch(mode="select",selected=None,component=None)            
        if editMode=="objects":
            if key==KEY.N:
                associatedimg = raw_input("image:")
                if len(associatedimg)==0:
                    associatedimg = "circle.png"
                md.objects.append(Bunch(name="",position=(100,100),pivot=(0,0),img=associatedimg))
                if associatedimg not in objimages:
                    objimages[associatedimg] = pyglet.resource.image(associatedimg)                    
                editInfo.mode='edit'
                editInfo.selected = md.objects[-1]
            if key==KEY.C:
                newimg = raw_input("change image:")
                if len(newimg)==0:
                    newimg = "circle.png"
                editInfo.selected.img = newimg
                objimages[newimg] = pyglet.resource.image(newimg)                
            if key==KEY.P:
                editInfo.mode='edit'
                editInfo.component='pos'
            if key==KEY.B:
                editInfo.mode='edit'
                editInfo.component='pivot'
            if key==KEY.I:
                id = raw_input("enter name/id:")
                print("You entered (" + id + ")")
                editInfo.selected.name = id            
                editInfo.mode = 'select'
            if key==KEY.X:
                print("deleting object "+editInfo.selected.name)
                md.objects.remove(editInfo.selected)
                editInfo.selected = None
                editInfo.mode='select'      
                                                                                                                                                                 
                       
        if key==KEY.Q:
            print("resetting edit mode")
            if editMode!="none":
                editMode = "none"
                
        if key==KEY.S:
            # save all current information
            print("Saving Map Data")
            md.output()
            pickler = pickle.Pickler(open(os.path.join("..","res",filename),"w"))
            pickler.dump(md)
            
    @window.event
    def on_draw():
        global t
        t += dt
        
        window.clear()   
        glColor3f(1,1,1)   
        bgimage.blit(0,0) 
        
        # draw annotations (red)
        if md.walkArea and editMode!="walkarea":
            glColor3f(1,0,0)
            glLineWidth(2)
            glBegin(GL_LINE_LOOP)
            for x,y in md.walkArea:
                glVertex2f(x,y)
            glEnd()
        
        glColor4f(0.5,.5,.5,.2)
        for s in md.interactionAreas:            
            glRectf(s.x-s.r,s.y-s.r,s.x+s.r,s.y+s.r)
        glColor4f(1,1,1,1)
        
#===============================================================================
#        for (lines,_) in md.walkAreaLayerSpecs:
#            glColor3f(1,1,0)
#            glLineWidth(2)
#            glBegin(GL_LINE_LOOP)
#            for x,y in lines:
#                glVertex2f(x,y)
#            glEnd()
#===============================================================================
            
        glColor3f(1,1,1)
        for obj in md.objects:
            img = objimages[obj.img]
            glColor4f(1,1,1,.2)
            glRectf(obj.position[0],obj.position[1],obj.position[0]+img.width,obj.position[1]+img.height)            
            glColor4f(1,1,1,1)
            apply(img.blit,obj.position)
            
            glPointSize(10)
            glColor3f(1,0,0)
            glBegin(GL_POINTS)
            dp = obj.pivot[0] + obj.position[0], obj.pivot[1] + obj.position[1]
            apply(glVertex2f,dp)
            glEnd()
        
                            
        # draw edit mode stuff
        
        if editMode=="walkarea":
            if editInfo.mode=='first':
                pass
            elif editInfo.mode=='adding':
                # draw the current polygon
                lines = copy(editInfo.points)
                lines.append((mx,my))                
                glLineWidth(5)
                glColor3f(0,0,1)
                glBegin(GL_LINES)
                for i in range(len(lines)-1):
                    apply(glVertex2f,lines[i])
                    apply(glVertex2f,lines[i+1])               
                glEnd()
        elif editMode=='interactionarea':
            if editInfo.mode=='select':
                pass
            elif editInfo.mode=='edit':
                s = editInfo.selected
                # draw this interaction area, as a rectangle
                glColor3f(0.5,1,.5)
                glRectf(s.x-s.r,s.y-s.r,s.x+s.r,s.y+s.r)
#===============================================================================
#        elif editMode=="walkarealayer":
#            if editInfo.mode=="adding":
#                # draw the current polygon
#                lines = copy(editInfo.selected[0])
#                if len(lines)>0:
#                    lines.append((mx,my))
#                    glLineWidth(5)
#                    glColor3f(0,0,1)
#                    glBegin(GL_LINES)
#                    for i in range(len(lines)-1):
#                        apply(glVertex2f,lines[i])
#                        apply(glVertex2f,lines[i+1])               
#                    glEnd()
#===============================================================================
        elif editMode=="objects":
            if editInfo.mode=="edit" or editInfo.mode=='selected':
                obj = editInfo.selected
                glColor4f(1,1,0,1)                
                apply(objimages[editInfo.selected.img].blit,editInfo.selected.position)
                glPointSize(10)
                glColor3f(1,0,0)
                glBegin(GL_POINTS)
                dp = obj.pivot[0] + obj.position[0], obj.pivot[1] + obj.position[1]
                apply(glVertex2f,dp)
                glEnd()                                        
                        
    pyglet.clock.schedule_interval(lambda dt: None, dt)
    window.set_visible(True)
    pyglet.app.run()  