'''
Created on 28/04/2009

Notes: Can build the MAP information offline and just load it

@author: ben
'''

#from Kitchen import *
#from BlobRoom import *
import room
from rooms import *
BlankRoom = room.Room.BlankRoom

from characters import (Character,Player,TheMan)
from TextRenderer import TextTransform

from euclid import *
from copy import copy

# Unique code from http://code.activestate.com/recipes/502263/
# Courtesy of Paul Rubin, and Jordan Callicoat
def unique(seq, keepstr=True):
  t = type(seq)
  if t in (str, unicode):
    t = (list, ''.join)[bool(keepstr)]
  seen = []
  return t(c for c in seq if not (c in seen or seen.append(c)))


class Map(object):
    "Specifies the structure of the world, and provides convenience functions for accessing routes, etc."
    
    def __init__(self):
        "only one map"

        self.rooms = {"front": Front(),
                      "side": Side(),
                      "back": Back(),
                      "shed": Shed(),
                      "catroom": CatRoom(),
                      "kitchen": Kitchen(),
                      "hallway": Hallway(),
                      "bathroom": Bathroom(),
                      "bedroom": Bedroom(),
                      "secretroom": SecretRoom(),
                      "loungeroom": LoungeRoom()}        
        
        # connection info (r1r2p1p2p3p4) r1p1->r2p2, r2p3->r1p4
        ci = [
              ("front","side",0,1,0,1),
              ("side","back",2,1,0,3),
              ("back","shed",4,1,0,5),
              ("back","catroom",2,1,0,3),
              ("catroom","kitchen",4,1,0,5),
              ("catroom","hallway",2,1,0,3),
              ("hallway","bathroom",2,1,0,3),
              ("hallway","bedroom",4,1,0,5),
              ("hallway","loungeroom",6,1,0,7),
              ("hallway","secretroom",8,1,None,None)]
        
        self.roomlist = self.rooms.values()
        
        # convert ci to a directed tree for the initilisation
        self.tree = []
        for (r1n,r2n,p1,p2,p3,p4) in ci:
            r1, r2 = self.rooms[r1n], self.rooms[r2n]
            if p1 is not None:
                self.tree.append((r1,r2,p1,p2))
            if p3 is not None:
                self.tree.append((r2,r1,p3,p4))
        
#===============================================================================
#        kitchen = Kitchen()
#        blankRoom1 = BlobRoom()
#        backDoorRoom = Room.Room.BlankRoom("The back room", "backdoorroom.dat")
#        self.roomMap = {"blobRoom":blankRoom1,"backDoorRoom":backDoorRoom,"kitchen":kitchen}
#        self.rooms = self.roomMap.values()
#        
#        # specify connectivity information
#        # (r1,r2,p1,p2) travel from r1 through p1 will take you to portal p2 in room r2
#        self.tree = [
#                    (blankRoom1,backDoorRoom,0,3),
#                    (backDoorRoom,blankRoom1,2,1),
#                    (backDoorRoom,kitchen,0,1),
#                    (kitchen,backDoorRoom,0,4)]
#        
#===============================================================================
        self.initialise()
        
    def initialise(self):
        "set up the route information and portals"
        # connect up portals
        for (r1,r2,p1,p2) in self.tree:
            #print r1.name,r2.name,p1,p2           
            r1.getPortal(p1).connect(r2.getPortal(p2)) 
        
        # traverse the tree and set up route info
        # for each node in the tree
        # trace a path to every other node
        self.route = {}
        
        for room in self.roomlist:
            # for each adjacent room, make a list of the rooms accessible from them
            for (adj,p) in self.allAdjacent(room):
                # build a list of all rooms accessible from adj                
                nodes_visited = [room,adj]
                leafnodes = self.allAdjacent(adj)
                if room in leafnodes:
                    leafnodes.remove(room)
                while len(leafnodes)>0: 
                    leafcopy = copy(leafnodes)
                    leafnodes = []
                    for (l,_) in leafcopy:
                        if l not in nodes_visited:
                            nodes_visited.append(l)
                            leafnodes.extend(self.allAdjacent(l))
                    leafnodes = unique(leafnodes)
                    for n in nodes_visited:
                        if n in leafnodes:
                            leafnodes.remove(n)
                nodes_visited.remove(room)
                for n in nodes_visited:
                    self.route[(room,n)] = p
        
    def adjacent(self,r1,r2):
        "returns the portal linking r1 to r2, or None if none"
        for (sr1,sr2,p1,_) in self.tree:
            if sr1==r1 and sr2==r2: return p1
        return None
        
    def allAdjacent(self,r1):
        "returns pairs of all adjacent rooms and portals"
        return [(sr2,p1) for (sr1,sr2,p1,_) in self.tree if sr1==r1]        
    
    def planRoute(self,roomfr,roomto):
        "returns a route to travel from roomfr to roomto"
        if roomfr==roomto: return None
        else: return self.route[(roomfr,roomto)]
    
class World(object):
    '''
    The game world
    '''
    # The only instance
    only = None
    
    def __init__(self):
        '''
        Constructs the entire game world in its initial state.
        '''
        World.only = self
 
        self.map = Map()
        self.rooms = self.map.roomlist
        
        self.player = Player(self.map.rooms["front"],Point2(200,100))
        self.theman = TheMan(self.map.rooms["loungeroom"],Point2(300,200))
        self.characters = [self.player, self.theman]
        
        self.localUpdates = [self.theman]
        self.globalUpdates = [self.theman]
        
        # cycle through object lists and add the appropriate ones to the local and global update lists
        for r in self.rooms:
            for o in r.objects:
                if o.localUpdateNeeded:
                    self.localUpdates.append(o)
                if o.globalUpdateNeeded:
                    self.globalUpdates.append(o)
                    
    def room(self,name):
        return self.map.rooms[name]
                    
        # print out plans for debugging purposes
#===============================================================================
#        for r1 in self.rooms:
#            for r2 in self.rooms:
#                print "From " + r1.name + " To " + r2.name + " Take Portal: ", self.map.planRoute(r1,r2)
#                
#===============================================================================
                                    
    def localUpdate(self,dt,room):
        "animation/fast update"
        self.player.localUpdate(dt)
        
        for o in self.localUpdates:
            if o.room==room:
                o.localUpdate(dt)
        
    def globalUpdate(self,dt):
        "slower, all object update"
        for o in self.globalUpdates:
            o.globalUpdate(dt)
            
    def planRoute(self,roomfr,roomto):
        "returns a route to travel from roomfr to roomto"
        return self.map.planRoute(roomfr, roomto)
        
        
        