'''
Created on 27/04/2009

Load a scene and test a walking character

@author: ben
'''

import pyglet
from pyglet.gl import *
import pyglet.window.key as KEY

from euclid import *
from copy import copy
import math, os
from random import random

from misc import *
from World import World
from RoomRenderer import RoomRenderer

from TextRenderer import (TextRenderer, TextTransform)

if __name__ == '__main__':
    # set up resource info
    pyglet.resource.path = ['../res']
    pyglet.resource.reindex()

    width, height = 800, 600
    window = pyglet.window.Window(width, height, visible=False)
    #window = pyglet.window.Window(fullscreen=True,visible=False)
    glEnable(GL_BLEND)
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
    
    world = World()
    player = world.player
    fps_display = pyglet.clock.ClockDisplay()
    
    RoomRenderer.setRoom(player.room)
    RoomRenderer.debug = True

    # 
    t = 0
    display_update = 1/15.0
    world_local_update = display_update
    world_global_update = 1/10.

    mx, my = 0, 0
    @window.event
    def on_mouse_motion(x,y,dx,dy):
        global mx,my
        mx,my = x,y
        
    @window.event
    def on_mouse_press(x,y,button,mod):
        print ("mouse click: ", (x,y))
            
    @window.event
    def on_key_press(key,mod):
        global debugdraw
        if key==KEY.D:
            RoomRenderer.debug = not RoomRenderer.debug
        if key==KEY.SPACE:
            player.room.use()
        else:
            walkingDirection = Dir.dirFromKey(key)
            if walkingDirection==None:
                player.stopWalking()
            else:
                player.startWalking(walkingDirection)
            
    @window.event
    def on_key_release(key,mod):
        if player.isWalking and player.walkingDirection==Dir.dirFromKey(key):
            player.stopWalking()  
    
    def update(dt):
        world.localUpdate(dt)
        
    def globalUpdate(dt):
        world.globalUpdate(dt)
        
    @window.event
    def on_draw():
        global t
        t += display_update
        #TODO: this should only change rarely..
        RoomRenderer.setRoom(player.room)
        
        window.clear()
        RoomRenderer.render(world.characters)
        
        fps_display.draw()

    pyglet.clock.schedule_interval(lambda dt: None, display_update)
    pyglet.clock.schedule_interval(update, world_local_update)
    #pyglet.clock.schedule_interval(globalUpdate, world_global_update)
    window.set_visible(True)    
    pyglet.app.run()  
    